"""Autogenerated API"""

import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)



@register_command(
    extending=("currentuser","v1","password"),
    module=argus_cli_module
)
def set_properties(
    oldPassword: str = None,
    newPassword: str = None,
    
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Change current users Argus password. Note that this will only change the password used for SMS, PASSWORD and TOTP, not external authentications like LDAP or RADIUS. (DEV)
    :param str oldPassword: Existing password, to validate password change 
    :param str newPassword: New password to change to. 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises InvalidPasswordException: on 412
    :returns: A requests object or JSON object
    """
    from os import getenv
    from requests import put
    from argus_api.exceptions import http
    
    from argus_cli.settings import settings
    
    url = "{}/currentuser/v1/password".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send oldPassword if the argument was provided, dont send null values
    if oldPassword is not None:
        body.update({ "oldPassword": oldPassword })
    # Only send newPassword if the argument was provided, dont send null values
    if newPassword is not None:
        body.update({ "newPassword": newPassword })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    if response.status_code == 401:
        raise http.AuthenticationFailedException(response)
    elif response.status_code == 403:
        raise http.AccessDeniedException(response)
    elif response.status_code == 412:
        raise http.ValidationErrorException(response)
    elif response.status_code == 404:
        raise http.ObjectNotFoundException(response)

    return response.json() if json else response
